"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withinFixture = exports.registerSelectorsFixture = exports.queriesFixture = exports.options = exports.installTestingLibraryFixture = void 0;
const test_1 = require("@playwright/test");
const helpers_1 = require("./helpers");
const defaultConfig = {
    asyncUtilExpectedState: 'visible',
    asyncUtilTimeout: 1000,
    testIdAttribute: 'data-testid',
};
const options = Object.fromEntries(Object.entries(defaultConfig).map(([key, value]) => [key, [value, { option: true }]]));
exports.options = options;
const queriesFixture = ({ page, asyncUtilExpectedState, asyncUtilTimeout }, use) => __awaiter(void 0, void 0, void 0, function* () { return use((0, helpers_1.queriesFor)(page, { asyncUtilExpectedState, asyncUtilTimeout })); });
exports.queriesFixture = queriesFixture;
const withinFixture = ({ asyncUtilExpectedState, asyncUtilTimeout }, use) => __awaiter(void 0, void 0, void 0, function* () {
    return use((locator) => (0, helpers_1.queriesFor)(locator, { asyncUtilExpectedState, asyncUtilTimeout }));
});
exports.withinFixture = withinFixture;
const engine = () => ({
    query(root, selector) {
        const args = JSON.parse(selector, window.__testingLibraryReviver);
        if ((0, helpers_1.isAllQuery)(queryName))
            throw new Error(`PlaywrightTestingLibrary: the plural '${queryName}' was used to create this Locator`);
        // @ts-expect-error
        const result = window.TestingLibraryDom[queryName](root, ...args);
        return result;
    },
    queryAll(root, selector) {
        const testingLibrary = window.TestingLibraryDom;
        const args = JSON.parse(selector, window.__testingLibraryReviver);
        // @ts-expect-error
        const result = testingLibrary[queryName](root, ...args);
        if (!result)
            return [];
        return Array.isArray(result) ? result : [result];
    },
});
const registerSelectorsFixture = [
    ({}, use) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield Promise.all(helpers_1.synchronousQueryNames.map((name) => __awaiter(void 0, void 0, void 0, function* () {
                return test_1.selectors.register((0, helpers_1.queryToSelector)(name), `(${engine.toString().replace(/queryName/g, `"${name}"`)})()`);
            })));
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.error('PlaywrightTestingLibrary: failed to register Testing Library functions\n', error);
        }
        yield use();
    }),
    { scope: 'worker', auto: true },
];
exports.registerSelectorsFixture = registerSelectorsFixture;
const installTestingLibraryFixture = [
    ({ context, asyncUtilExpectedState, asyncUtilTimeout, testIdAttribute }, use) => __awaiter(void 0, void 0, void 0, function* () {
        yield context.addInitScript(yield (0, helpers_1.buildTestingLibraryScript)({
            config: { asyncUtilExpectedState, asyncUtilTimeout, testIdAttribute },
        }));
        yield use();
    }),
    { scope: 'test', auto: true },
];
exports.installTestingLibraryFixture = installTestingLibraryFixture;
//# sourceMappingURL=fixtures.js.map