import type { Locator, Page } from '@playwright/test';
import type { AllQuery, Config, LocatorQueries as Queries, Query, Selector, SynchronousQuery } from '../types';
declare const isAllQuery: (query: Query) => query is AllQuery;
declare const queryToSelector: (query: SynchronousQuery) => Selector;
declare const buildTestingLibraryScript: ({ config }: {
    config: Config;
}) => Promise<string>;
declare const synchronousQueryNames: ("getByLabelText" | "getAllByLabelText" | "queryByLabelText" | "queryAllByLabelText" | "getByPlaceholderText" | "getAllByPlaceholderText" | "queryByPlaceholderText" | "queryAllByPlaceholderText" | "getByText" | "getAllByText" | "queryByText" | "queryAllByText" | "getByAltText" | "getAllByAltText" | "queryByAltText" | "queryAllByAltText" | "getByTitle" | "getAllByTitle" | "queryByTitle" | "queryAllByTitle" | "getByDisplayValue" | "getAllByDisplayValue" | "queryByDisplayValue" | "queryAllByDisplayValue" | "getByRole" | "getAllByRole" | "queryByRole" | "queryAllByRole" | "getByTestId" | "getAllByTestId" | "queryByTestId" | "queryAllByTestId")[];
/**
 * Given a `Page` or `Locator` instance, return an object of Testing Library
 * query methods that return a `Locator` instance for the queried element
 *
 * @internal this API is not currently intended for public usage and may be
 * removed or changed outside of semantic release versioning. If possible, you
 * should use the `locatorFixtures` with **@playwright/test** instead.
 * @see {@link locatorFixtures}
 *
 * @param pageOrLocator `Page` or `Locator` instance to use as the query root
 * @param config Testing Library configuration to apply to queries
 *
 * @returns object containing scoped Testing Library query methods
 */
declare const queriesFor: (pageOrLocator: Page | Locator, config?: Partial<Config> | undefined) => Queries;
export { buildTestingLibraryScript, isAllQuery, queriesFor, queryToSelector, synchronousQueryNames };
