"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.synchronousQueryNames = exports.queryToSelector = exports.queriesFor = exports.isAllQuery = exports.buildTestingLibraryScript = void 0;
const fs_1 = require("fs");
const test_1 = require("@playwright/test");
const dom_1 = require("@testing-library/dom");
const common_1 = require("../../common");
const helpers_1 = require("../helpers");
const allQueryNames = Object.keys(dom_1.queries);
const isAllQuery = (query) => query.includes('All');
exports.isAllQuery = isAllQuery;
const isFindQuery = (query) => query.startsWith('find');
const isNotFindQuery = (query) => !query.startsWith('find');
const findQueryToGetQuery = (query) => query.replace(/^find/, 'get');
const findQueryToQueryQuery = (query) => query.replace(/^find/, 'query');
const queryToSelector = (query) => query.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
exports.queryToSelector = queryToSelector;
const buildTestingLibraryScript = ({ config }) => __awaiter(void 0, void 0, void 0, function* () {
    const testingLibraryDom = yield fs_1.promises.readFile(require.resolve('@testing-library/dom/dist/@testing-library/dom.umd.js'), 'utf8');
    const configuredTestingLibraryDom = (0, common_1.configureTestingLibraryScript)(testingLibraryDom, config);
    return `
    ${configuredTestingLibraryDom}
    
    window.__testingLibraryReviver = ${helpers_1.reviver.toString()};
  `;
});
exports.buildTestingLibraryScript = buildTestingLibraryScript;
const synchronousQueryNames = allQueryNames.filter(isNotFindQuery);
exports.synchronousQueryNames = synchronousQueryNames;
const createFindQuery = (pageOrLocator, query, { asyncUtilTimeout, asyncUtilExpectedState } = {}) => (...[id, options, waitForElementOptions]) => __awaiter(void 0, void 0, void 0, function* () {
    const synchronousOptions = [id, options].filter(Boolean);
    const locator = pageOrLocator.locator(`${queryToSelector(findQueryToQueryQuery(query))}=${JSON.stringify(synchronousOptions, helpers_1.replacer)}`);
    const { state = asyncUtilExpectedState, timeout = asyncUtilTimeout } = waitForElementOptions !== null && waitForElementOptions !== void 0 ? waitForElementOptions : {};
    try {
        yield locator.first().waitFor({ state, timeout });
    }
    catch (error) {
        // In the case of a `waitFor` timeout from Playwright, we want to
        // surface the appropriate error from Testing Library, so run the
        // query one more time as `get*` knowing that it will fail with the
        // error that we want the user to see instead of the `TimeoutError`
        if (error instanceof test_1.errors.TimeoutError) {
            return pageOrLocator
                .locator(`${queryToSelector(findQueryToGetQuery(query))}=${JSON.stringify(synchronousOptions, helpers_1.replacer)}`)
                .first()
                .waitFor({ state, timeout: 100 });
        }
        throw error;
    }
    return locator;
});
/**
 * Given a `Page` or `Locator` instance, return an object of Testing Library
 * query methods that return a `Locator` instance for the queried element
 *
 * @internal this API is not currently intended for public usage and may be
 * removed or changed outside of semantic release versioning. If possible, you
 * should use the `locatorFixtures` with **@playwright/test** instead.
 * @see {@link locatorFixtures}
 *
 * @param pageOrLocator `Page` or `Locator` instance to use as the query root
 * @param config Testing Library configuration to apply to queries
 *
 * @returns object containing scoped Testing Library query methods
 */
const queriesFor = (pageOrLocator, config) => allQueryNames.reduce((rest, query) => (Object.assign(Object.assign({}, rest), { [query]: isFindQuery(query)
        ? createFindQuery(pageOrLocator, query, config)
        : (...args) => pageOrLocator.locator(`${queryToSelector(query)}=${JSON.stringify(args, helpers_1.replacer)}`) })), {});
exports.queriesFor = queriesFor;
//# sourceMappingURL=helpers.js.map